import _mapInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/map";
import _sortInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/sort";
import _filterInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/filter";
import _indexOfInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/index-of";
import _reduceInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/reduce";
import _Object$keys from "@babel/runtime-corejs3/core-js-stable/object/keys";
import _includesInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/includes";
/**
 * Filters React Select options and sorts by similarity to a search filter.
 * Handles partial matches, eg. searching for "Waberg High" will find "Raoul
 * Wallenberg Traditional High School". Case insensitive. Ignores
 * non-alphanumeric characters.
 *
 * @param  options  An unfiltered list of Options.
 * @param filter A string to compare against Option labels.
 * @param substitutions Strings with multiple spellings or variations that we
 *           expect to match, eg. accented characters or abbreviated words.
 *
 * @return A filtered and sorted array of Options.
 */
export function filterOptions(options, filter, substitutions) {
  var _context, _context2, _context3, _context4;
  // If the filter is blank, return the full list of Options.
  if (!filter) {
    return options;
  }
  const cleanFilter = cleanUpText(filter, substitutions);
  return _mapInstanceProperty(_context = _sortInstanceProperty(_context2 = _filterInstanceProperty(_context3 = _mapInstanceProperty(_context4 = _filterInstanceProperty(options
  // Filter out undefined or null Options.
  ).call(options
  // Filter out undefined or null Options.
  , _ref => {
    let {
      label,
      value
    } = _ref;
    return label != null && value != null;
  })
  // Create a {score, Option} pair for each Option based on its label's
  // similarity to the filter text.
  ).call(_context4, option => ({
    option: option,
    score: typeaheadSimilarity(cleanUpText(option.label, substitutions), cleanFilter)
  }))
  // Only include matches of the entire substring, with a slight
  // affordance for transposition or extra characters.
  ).call(_context3, pair => pair.score >= Math.max(cleanFilter.length - 2, cleanFilter.length) /** Filter text might have length less than 2 */)
  // Sort 'em by order of their score.
  ).call(_context2, (a, b) => b.score - a.score)
  // …and grab the original Options back from their pairs.
  ).call(_context, pair => pair.option);
}
/**
 * Scores the similarity between two strings by returning the length of the
 * longest common subsequence. Intended for comparing strings of different
 * lengths; eg. when matching a typeahead search input with a school name.

 * Meant for use in an instant search box where results are being fetched
 * as a user is typing.
 *
 * @param  a  The longer string (though, we flip them if it's shorter).
 * @param  b  The shorter string, eg. a typeahead search input.
 *
 * @return The length of the longest common subsequence. Higher scores indicate
 *           closer matches.
 */
export function typeaheadSimilarity(a, b) {
  let aLength = a.length;
  let bLength = b.length;
  const table = [];
  if (!aLength || !bLength) {
    return 0;
  }
  // Ensure `a` isn't shorter than `b`.
  if (aLength < bLength) {
    [a, b] = [b, a];
    // Swap the length of these strings.
    [aLength, bLength] = [bLength, aLength];
  }
  // Early exit if `a` includes `b`; these will be scored higher than any
  // other options with the same `b` (filter string), with a preference for
  // shorter `a` strings (option labels).
  if (_indexOfInstanceProperty(a).call(a, b) !== -1) {
    return bLength + 1 / aLength;
  }
  // Initialize the table axes:
  //
  //    0 0 0 0 ... bLength
  //    0
  //    0
  //
  //   ...
  //
  // aLength
  //
  for (let x = 0; x <= aLength; ++x) {
    table[x] = [0];
  }
  for (let y = 0; y <= bLength; ++y) {
    table[0][y] = 0;
  }
  // Populate the rest of the table with a dynamic programming algorithm.
  for (let x = 1; x <= aLength; ++x) {
    for (let y = 1; y <= bLength; ++y) {
      table[x][y] = a[x - 1] === b[y - 1] ? 1 + table[x - 1][y - 1] : Math.max(table[x][y - 1], table[x - 1][y]);
    }
  }
  return table[aLength][bLength];
}
/**
 * Apply string substitutions, remove non-alphanumeric characters, and convert
 * all letters to uppercase.
 *
 * eg. 'Scoil Bhríde Primary School' may become 'SCOILBHRIDEPRIMARYSCHOOL'.
 *
 * @param  input  An unsanitized input string.
 * @param  substitutions  Strings with multiple spellings or variations that we
 *          expect to match, for example accented characters or abbreviated
 *          words.
 *
 * @return The sanitized text.
 */
export function cleanUpText(input, substitutions) {
  var _context5;
  if (!input) {
    return '';
  }
  // Uppercase and remove all non-alphanumeric, non-accented characters.
  // Also remove underscores.
  input = input.toUpperCase().replace(/((?=[^\u00E0-\u00FC])\W)|_/g, '');
  if (!substitutions) {
    return input;
  }
  const safeSubstitutions = substitutions; // For Flow.
  // Replace all strings in `safeSubstitutions` with their standardized
  // counterparts.
  return _reduceInstanceProperty(_context5 = _Object$keys(safeSubstitutions)).call(_context5, (output, substitution) => {
    const unsubbed = new RegExp(substitution, 'g');
    return output.replace(unsubbed, safeSubstitutions[substitution]);
  }, input);
}
export function simpleFilterOptions(options, filter) {
  // If the filter is blank, return the full list of Options.
  if (!filter) {
    return options;
  }
  // Simple sub-string match
  return _filterInstanceProperty(options).call(options, _ref2 => {
    var _context6;
    let {
      label,
      value
    } = _ref2;
    return label != null && value != null && _includesInstanceProperty(_context6 = label.toLowerCase()).call(_context6, filter.toLowerCase());
  });
}